;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x161281ea9ac2fee410235ee5c0ed283e . "./code.cg2js/cg-status-bar.fasl")
(#x07c8d1337a633d1c80bc134a730b8ed4 . "./code.cg2js/cg-hotspot.fasl")
(#xd39285342be1f57fba54057947a188ed . "./code.cg2js/cg-toolbar.fasl")
(#x9230576336577e19e0d780bd5d310c1e . "./code.cg2js/cg-clipboard-stack.fasl")
(#xe223cbb67163b5af6be7bd11d93a2015 . "./code.cg2js/cg-tooltip.fasl")
(#x43a3dc26919c877c5a71e37707229e91 . "./code.cg2js/cg-calendar.fasl")
(#x645caef33a44595895da1e76497268c9 . "./code.cg2js/cg-item-list.fasl")
(#x2ddef344d9493bfe767294dbbeaedd37 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xb6ec6e1de8e9163e0c359286b2f6014f . "./code.cg2js/cg-trackbar.fasl")
(#x7022d6c16ca25c59670d52687b70b6ec . "./code.cg2js/cg-class-grid.fasl")
(#xba1a4a1022d24a4796e93293b9964650 . "./code.cg2js/cg-dialog-item.fasl")
(#xdf98ae9a74053c7325a959a39f45d66a . "./code.cg2js/cg-comtab.fasl")
(#xc4ce22c75de2121b31a9e741044195b2 . "./code.cg2js/cg-lisp-text.fasl")
(#x689ce07cf0151af8914741bbc62d1883 . "./code.cg2js/cg-button.fasl")
(#x232793ad865d7afb813612f4aad0623d . "./code.cg2js/cg-printing.fasl")
(#xf7624f8a6a331d627d15971f89ca9d23 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x2af9e6d89256d3f5f7acb4973fc03412 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x5ff22b132b5b9922fde2211c24c0bfab . "./code.cg2js/cg-pixmap-widget.fasl")
(#xf5cc35c76979741896a593d976a96ae4 . "./code.cg2js/cg-selected-object.fasl")
(#xd41037e80e67712a8490ef15dda25b61 . "./code.cg2js/cg-text-or-combo.fasl")
(#x5aaba68844216a6766106655965c0be6 . "./code.cg2js/cg-graphics-context.fasl")
(#x2c692561ecb105bf58c0f2d9d03ed566 . "./code.cg2js/cg-edit-in-place.fasl")
(#xa71b9a3973da83e8af94affddde82cb2 . "./code.cg2js/cg-string-dialog.fasl")
(#xaa7925becbcb9af7e009da3b94b19ef2 . "./code.cg2js/cg-list-view.fasl")
(#x71f7b4e163a7e4b851f2abdfdfb0e048 . "./code.cg2js/cg-intersect.fasl")
(#x69b4e88f2313338de12f7d05d6af9da5 . "./code.cg2js/cg-text-widget.fasl")
(#x073844d5429f00df35f92997db541cc8 . "./code.cg2js/cg-directory-dialog.fasl")
(#xd9c7d6cfc52d14f47d6340f8e22e09f7 . "./code.cg2js/cg-file-dialog.fasl")
(#x73ca497f8423b5935e6d9a5da49c0d27 . "./code.cg2js/cg-dde-utils.fasl")
(#x2ca552407a235dcd03609074784f6429 . "./code.cg2js/cg-nodes.fasl")
(#xacb420f8a0fdedfaf65f324681d8e91b . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x22fb1aa7702ae2249aad5bd6f781f913 . "./code.cg2js/cg-up-down-control.fasl")
(#xa5af7ef586c8168f81bc74f5cd115542 . "./code.cg2js/cg-get-position.fasl")
(#xdac0f2dff42b589b3d1b5743b6d2848c . "./code.cg2js/cg-toggling-widget.fasl")
(#x47628e4adf753b38432ace3e577d0ef6 . "./code.cg2js/cg-acache.fasl")
(#x7ee6326ba8a640da82526485f36d2fcd . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xa3b7d6e75ab7a3d692cb809bec300c4b . "./code.cg2js/cg-base.fasl")
(#x54bf5d70ed0e2b7fff46bbad0287a33d . "./code.cg2js/cg-drag-and-drop.fasl")
(#x875f6097b1d1083324b19c6b23ad4d33 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xae2421e64e93c5161bc782df3a7fe747 . "./code.cg2js/cg-property.fasl")
(#x38718f771ca652a32440b78ecc4a1fa4 . "./code.cg2js/cg-editable-text.fasl")
(#x542993bf172de831a02efbc577861900 . "./code.cg2js/cg-lamp.fasl")
(#xefef69614ff9a822f537bf1f904da7e7 . "./code.cg2js/cg-progress-indicator.fasl")
(#x042414e883ad2c6fa276f177b475bb19 . "./code.cg2js/cg-chart-widget.fasl")
(#x7e17dff3fc22c7db4b65a1b897ee79de . "./code.cg2js/cg-common-control.fasl")
(#xeaec86dfc3d671efe22b661bd7d5d2dc . "./code.cg2js/cg-font-dialog.fasl")
(#x37d24ee091fb6c8a478a5957467c5e65 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x4848b6d0b394e0b0ade6120e39e4e555 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xabf03f9526ec64e2efb40a57c465522c . "./code.cg2js/cg-os-window.fasl")
(#x217734426a45db266a5f8f80c4806a85 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x7b655e8aa5759291fbf8be3a7fdd47ae . "./code.cg2js/cg-pixmap.fasl")
(#xccef9f1e209c3fd5e95f59cd1aca067f . "./code.cg2js/cg-chart-or-plot.fasl")
(#xc2188460b23a633e2dd41cf2abf4ff67 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x53cd0bfc7f5448d0f7d7f33800553f5e . "./code.cg2js/cg-scaling-stream.fasl")
(#xc31a8df6c476afba3744c28f3eb6837e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x1bf09b3cf053d780a25d6a3d5def215d . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x6068ddc5304851940dfb75d1db73dfff . "./code.cg2js/cg-timer.fasl")
(#xa8322818038ddbb7d4183ea68eb1f9cd . "./code.cg2js/cg-scroll-bar.fasl")
(#x4c7ca480548335bbf95940a7c642e4de . "./code.cg2js/cg-caret.fasl")
(#xa82374b14d553c8c4a947c0307324a96 . "./code.cg2js/cg-template-string.fasl")
(#xb68009763203860323a97ea4f850de40 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xcad3aabddb2039ba1b1bd1a5ccad38d9 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x089fe25ff57e3c44d220a39940273f31 . "./code.cg2js/cg-combo-box.fasl")
(#x420a2b712d4c663fd87d0b3aed13ac5c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x2724a941792eb5eee8503aa3422f8c3c . "./code.cg2js/cg-multi-picture-button.fasl")
(#xed6dc59771374a90482deb8a965c1c34 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x2d2c1ffc702354a34c987a810025aa1d . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x59205653528de661eaafd308b0cae2aa . "./code.cg2js/cg-player.fasl")
(#x8a4e33fec21df86b6e5bd7c7e0f3b266 . "./code.cg2js/cg-outline.fasl")
(#x78e0e35da0fda102a9160d1eb270912f . "./code.cg2js/cg-class-support.fasl")
(#xb4a4623356f32b1c2f30a15b0d67110b . "./code.cg2js/cg-lettered-menu.fasl")
(#xa8dd5d902bbd6516e6de45f6360ea846 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x12efd2bb07dd3484cad89fc021be0568 . "./code.cg2js/cg-fill-texture.fasl")
(#x50e68dbc32674c14ab3d6f8440997eab . "./code.cg2js/cg-lisp-widget.fasl")
(#x5404144e20c842d291bf43961d8d599c . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xed1e1d44ea8ee8e0a708a084826baf8f . "./code.cg2js/cg-clipboard.fasl")
(#xd0cb5bb38a657ac1c46ef1b1410b55ac . "./code.cg2js/cg-wrap-string.fasl")
(#x6cbaf82f8978f6f60a10011363ace507 . "./code.cg2js/cg-os-widget.fasl")
(#xa91ab928f098f8ad2602343b263bfb08 . "./code.cg2js/cg-check-box.fasl")
(#x85284b77eb859112cc48b5a893bf5af4 . "./code.cg2js/cg-choice-list.fasl")
(#x1d3d5ba9aa85e0c2abe1d1b1f8576950 . "./code.cg2js/cg-layout.fasl")
(#x18819df3531883e45394281fe08b21af . "./code.cg2js/cg-group-box.fasl")
(#x8f43af515b8fa4a8516ec653b96a1651 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xb60e3fc26185eb6bd9988a27579e786e . "./code.cg2js/cg-radio-button.fasl")
(#xf2a3a3d94e99c02b3da80a3ac2cd4bc7 . "./code.cg2js/cg-tab-control.fasl")
(#x300f354cb763b09d7cbccf535c1459a8 . "./code.cg2js/cg.fasl")
(#x18c3106b59b6499ccc8a6f38f2662091 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x01e5f8be5fbd16c87288943d8c0bc0ae . "./code.cg2js/cg-dropping-outline.fasl")
(#xcf5734b9236eb883418a7786847320f5 . "./code.cg2js/cg-get-pixmap.fasl")
(#xff79d54754ec6a6f7e52c186e57fb21e . "./code.cg2js/cg-grid-widget.fasl")
(#xf3e06bd45eb331e2f1b4233ccfb9f9ea . "./code.cg2js/cg-utility-dialog.fasl")
(#x91109c504c91f76f22fa4a3d986f67ab . "./code.cg2js/cg-static-text.fasl")
(#x9fe108587ece649165e6c157f919f6d9 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x3e7b85874b81da8137c716fa3ea14240 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x6f63d86b53d1a628975ba479d26db997 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc8209eaac7b53c81db786ec844d6e92c . "./code.cg2js/cg-bitmap-pane.fasl")
(#xb44bf53f53267e0d5abfd6ace7b382f3 . "./code.cg2js/cg-plot-widget.fasl")
(#x9bb16e738084977a2d3f4aa117e1c405 . "./code.cg2js/cg-color-dialog.fasl")
(#x6defc1463fae58ce1b7537a0c53e669d . "./code.cg2js/cg-icon-pixmap.fasl")
(#xfc57463320b13cf0f29367623d3210c6 . "./code.cg2js/cg-icon.fasl")
(#x006646d0a103a9ae6db6f8690774d76b . "./code.cg2js/cg-dev.fasl")
(#xaeba119f1ff5b509517587944d27dd1f . "./code.cg2js/cg-choose-printer.fasl")
(#xb49b22fa79cb613ea31254c178682bdd . "./code.cg2js/cg-intersect-posbox.fasl")
(#x8e2d1e60b9d545b875e50c533afae69c . "./code.cg2js/cg-menu.fasl")
(#x356e21dc574539c861221516f6a4be40 . "./code.cg2js/cg-yes-no-list.fasl")
(#xfab030217ddc0eb46623042d0418f2c0 . "./code.cg2js/cg-palette.fasl")
(#xba20cf3414487596563d8bd131916c97 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xaebb0c52da11e747922968b3e8f5d27e . "./code.cg2js/cg-message-dialog.fasl")
(#xd210b4d7af3d23fb7a5074be2dc8eb9d . "./code.cg2js/cg-shortcut-menu.fasl")
(#x6374d353dfc6a182a4e39832ece2ce15 . "./code.cg2js/cg-split-bar.fasl")
(#x2041746231419c5ea866d907b38de820 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xb836f9e5c38005800cfe3d0a9c85be78 . "./code.cg2js/cg-web-browser.fasl")
(#xca4c39b0ddfdccf3e76a1868cd5440fb . "./code.cg2js/cg-project-window.fasl")
(#xb7a573cb5274c031e32b2de50e57d9e9 . "./code.cg2js/cg-picture-widget.fasl")
(#xaefb747fb9cc27df7e1a4d1eead49e8b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x2094ca56007d76374ae51a7474fc80df . "./code.cg2js/cg-object-editor.fasl")
(#x3345d391c2ed1ca16c3b3254e2c7a17d . "./code.cg2js/cg-paren-matching.fasl")
(#xd2a9b1d7b85f99cf009691ca8b83f107 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xad4ce3f2200c7eb42980734cd093bdf5 . "./code.cg2js/cg-drawable.fasl")
(#x642308600f70ac884dfc0deae70adf52 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xa850d7c4ce919bca517050d0dc487c7b . "./code.cg2js/cg-html-widget.fasl")
(#x381d8e07fcdb4ffa2ae6022b47d85fa9 . "./code.cg2js/cg-curve.fasl")
(#x04b23b095fc6eb3f4360f8dd8504d672 . "./code.cg2js/cg-yes-no-string.fasl")
(#xf23da5b1567e906ca514e86605bb31f8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x76566d7d11c0ab6a113ba8a5b06f7fbb . "./code.cg2js/cg-html-dialog.fasl")
(#x3564811b2c167d7c1871bee4839da66a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x72a660addb4868da4dbbb861b700bde2 . "./code.cg2js/cg-tray.fasl")
(#xa886cb043d7dcfe83c1f80b335e5dda9 . "./code.idejs/ide-apropos.fasl")
(#x0d72138243453abafd0c71ff480072cf . "./code.idejs/ide-window-list.fasl")
(#x883ee594481e6b423b6d322f548ddca0 . "./code.idejs/ide-class-browser.fasl")
(#x319e9e523780e8958b86b7189b0c0949 . "./code.idejs/ide-base.fasl")
(#xfdbbed930ef44fe78df35dd9a3dd4d92 . "./code.idejs/ide-process-browser.fasl")
(#x3ea1fc6fd5d2656e131b02d348d4d034 . "./code.idejs/ide-acache.fasl")
(#x13b809ebc4de9f63ea58edbc921af8c6 . "./code.idejs/ide-navigator.fasl")
(#xb06706fe2d5e0e3c99a0717d8a6aaae7 . "./code.idejs/ide-trace.fasl")
(#x5cfca76700d7758abdd4d04cee2ce481 . "./code.idejs/ide-project.fasl")
(#x560a9c847bf2a42f1a4068225d3c95af . "./code.idejs/ide-patch-dialog.fasl")
(#x81c862e3cc43d211393f596361f04830 . "./code.idejs/ide-grapher.fasl")
(#x9f0e11d2df18429f8580fc930a0ba083 . "./code.idejs/ide-dev.fasl")
(#xf22e9ff18e1a2a5046b56cf459947dd1 . "./code.idejs/ide-keyshort.fasl")
(#x0572b8c8e27fea94d062877c2f50a879 . "./code.idejs/ide-editor.fasl")
(#xab6dc8a32a7fa82874ec30344140a800 . "./code.idejs/ide-inspector.fasl")
(#xf44ccef07a3b6376864d6e822457d614 . "./code.idejs/ide.fasl")
(#xd5479673d12e4620c740d6289d555b07 . "./code.idejs/ide-clipboard.fasl")
(#x46a16ea4ed917e170c3f66efe32ecdec . "./code.idejs/ide-find-definitions.fasl")
(#xbf876177999911bee91768e01f83d2ad . "./code.idejs/ide-form.fasl")
(#x4a71a9848d592b8213f62abed0233aa1 . "./code.idejs/ide-intro.fasl")
(#x51a059079f710dfd5b9d211b4ed2e256 . "./code.idejs/ide-find-in-files.fasl")
(#x6f5aa15025501949eccda7a360d6f651 . "./code.idejs/ide-package-list.fasl")
(#x3a89e44d7a05e719c555c366b6a7e4ad . "./code.idejs/ide-class-grapher.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
